!$hmpp parabola codelet, target=CUDA, transfer=atcall, &
!$hmpp & args[para, a, deltaa, a6, al, flat].mirror, &
!$hmpp & args[para, a, deltaa, a6, al, flat].transfer=manual, &
!$hmpp & args[nmin, nmax].io=in
subroutine parabola( nmin, nmax, para, a, deltaa, a6, al, flat )

! Colella and Woodward, JCompPhys 54, 174-201 (1984) eq 1.5-1.8,1.10
!
! parabola calculates the parabolas themselves. call paraset first
! for a given grid-spacing to set the constants, which can be reused
! each time parabola is called.
!
! flatening coefficients are calculated externally in flaten. 
! nmin/nmax are indicies over which the parabolae are calculated
!-----------------------------------------------------------------------
! GLOBALS
use sweepsize

IMPLICIT NONE

! LOCALS
integer :: n, nmin, nmax
real :: onemfl
real, dimension(maxsweep) :: a, deltaa, a6, al, flat
real, dimension(maxsweep,5) :: para


real, dimension(maxsweep) ::  ar

real diffa_N_m1, diffa_N, diffa_N_p1, da_N, da_N_p1
real scrch1, scrch2, scrch3, tmp_ar, tmp_al

  do n = nmin-1, nmax

     diffa_N_p1 = a(n+2) - a(n+1)
     diffa_N    = a(n+1) - a(n+0)
     diffa_N_m1 = a(n+0) - a(n-1)

     da_N    = 0
     da_N_p1 = 0

     !!da(n)
     if(diffa_N_m1*diffa_N < 0.0) then
        da_N = 0.0
     else
        da_N = para(n,4) * diffa_N + para(n,5) * diffa_N_m1
        da_N = sign( min(abs(da_N), 2.0*abs(diffa_N_m1), 2.0*abs(diffa_N)), da_N )
     endif


     !!da(n+1)
     if(diffa_N*diffa_N_p1 < 0.0) then
        da_N_p1 = 0.0
     else
        da_N_p1 = para(n+1,4) * diffa_N_p1 + para(n+1,5) * diffa_N
        da_N_p1 = sign( min(abs(da_N_p1), 2.0*abs(diffa_N), 2.0*abs(diffa_N_p1)), da_N_p1 )
     endif


    ar(n) = a(n) + para(n,1)*diffa_N + para(n,2)*da_N_p1 + para(n,3)*da_N
!    ar(n) = max(ar(n),min(a(n),a(n+1)))
!    ar(n) = min(ar(n),max(a(n),a(n+1)))
    al(n+1) = ar(n)
  enddo


  do n = nmin, nmax
     onemfl= 1.0 - flat(n)
     tmp_ar = flat(n) * a(n) + onemfl * ar(n)
     tmp_al = flat(n) * a(n) + onemfl * al(n)


     deltaa(n) = tmp_ar - tmp_al
     a6(n)     = 6. * (a(n) - .5 * (tmp_al + tmp_ar))
     scrch1 = (tmp_ar - a(n)) * (a(n)-tmp_al) 
     scrch2 = deltaa(n) * deltaa(n)
     scrch3 = deltaa(n) * a6(n)

     if(scrch1 <= 0.0) then
        tmp_ar = a(n)
        tmp_al = a(n)
     endif
     if(scrch2 < +scrch3) tmp_al = 3. * a(n) - 2. * tmp_ar       
     if(scrch2 < -scrch3) tmp_ar = 3. * a(n) - 2. * tmp_al

     deltaa(n)= tmp_ar - tmp_al
     a6(n) = 6. * (a(n) - .5 * (tmp_al + tmp_ar))
     ar(n) = tmp_ar
     al(n) = tmp_al
  enddo

return
end

!#######################################################################


subroutine paraset( nmin, nmax, para, dx, xa )

! Colella and Woodward, JCompPhys 54, 174-201 (1984) eq 1.6, 1.7
!
! paraset sets up constants which are re-used each time we want to
! interpolate a parabola on a quantity. First pull out constants
! A, B, and C, and then compute all the equations in terms of those quantities. 
! The quantities calculated here are stored in a array para, to be read by parabola()
! nmin/nmax are index range for which one will calculate parabolae
!-----------------------------------------------------------------------------------
! GLOBALS
use sweepsize

IMPLICIT NONE

! LOCALS
integer :: n, nmin, nmax
real, dimension(maxsweep,5) :: para
real, dimension(maxsweep) :: dx, xa, a, b, c, d, ai, bi, ci

!------------------------------------------------------------------------------

do n = nmin-2, nmax+1
  a (n) = dx(n) + dx(n+1)
  ai(n) = 1.0/a(n)
  b (n) = a(n) + dx(n)
  bi(n) = 1.0/b(n)
  c (n) = a(n) + dx(n+1)
  ci(n) = 1.0/c(n)
enddo

!                                        constants for equation 1.6
!     a(j+.5) = a(j) + C1 * (a(j+1)-a(j)) + C2 * da(j+1) + C3 * da(j)

do n = nmin-1, nmax
  d(n)      = 1. / (a(n-1) + a(n+1))
  para(n,1) = dx(n) * ai(n) + 2. * dx(n+1) * dx(n) * d(n) * ai(n) * ( a(n-1) * bi(n) - a(n+1) * ci(n) )
  para(n,2) = - d(n) * dx(n)   * a(n-1) * bi(n)
  para(n,3) =   d(n) * dx(n+1) * a(n+1) * ci(n)
enddo

!                                        constants for equation 1.7
!     da(j) = D1 * (a(j+1) - a(j)) + D2 * (a(j) - a(j-1))

do n = nmin-1, nmax+1
  d(n) = dx(n) / ( a(n-1) + dx(n+1) )
  para(n,4) = d(n) * b(n-1) * ai(n)
  para(n,5) = d(n) * c(n)   * ai(n-1)
enddo

return
end

